import numpy as np
import sys
import math
import h5py
import cons_zeus
import os.path

lista=sys.argv[1:]
n= len(lista)

direc=''
if not os.path.isfile(direc+'hdffix_yt'):
        sys.exit('hdffix_yt not found')

#grab informations on the numebr of grids, indexes, etc...
hdffixyt = h5py.File(direc+'hdffix_yt','r')
n_patches=hdffixyt['n_patches'][()]
patches_dim = hdffixyt['patches_dim'][()]
r = hdffixyt['/grid/r_total'][()]
theta= hdffixyt['/grid/theta'][()]
phi= hdffixyt['/grid/phi'][()]
r_patch_index_initial = hdffixyt['r_patch_index_initial'][()]
r_patch_index_final = hdffixyt['r_patch_index_final'][()]
theta_dim=len(theta)
phi_dim=len(phi)


#n_patches x 3 array of np.int64 type
grid_dimensions_GDF = np.tile( \
        np.array([0,theta_dim, phi_dim]).astype(np.int64),(n_patches,1))
grid_dimensions_GDF[:,0] = patches_dim.astype(np.int64)

#----
#FIXME
#these are my guesses
grid_left_index_GDF = np.zeros((n_patches,3), dtype=np.int64)
grid_left_index_GDF[:,0] = r_patch_index_initial.astype(np.int64)

#this is an array from 0 to patches_dim-1
grid_level_GDF =  np.linspace(0, len(patches_dim), len(patches_dim), \
        endpoint=False).astype(np.int64)

# this DOES NOT work
#grid_level_GDF =  np.zeros(len(patches_dim), dtype=np.int64)
#----

#main loop
for i in range(0,n):

    hydroyt = h5py.File(direc+lista[i]+'yt',"r")
    time=hydroyt['time'][()]
    hydroyt.close()

    amr = h5py.File('amr_zeus'+str(i)+'.h5',"w")


    #create groups following the specifications of the GDF format in yt
    #these groups are mandatory, even if they are empty
    data=amr.create_group("data")
    #dataset_units = amr.create_group("dataset_units")
    amr.create_group("domains")
    field_types = amr.create_group("field_types")
    amr.create_group("particle_types")
    gridded_data_format = amr.create_group("gridded_data_format")
    simulation_parameters = amr.create_group("simulation_parameters")


    #this was included in the sample file of the GDF format
    data.attrs['cg_count']=n_patches

    #not sure if this number refers to GDF format...
    gridded_data_format.attrs['format_version']=1.1

    #we need some attributes to simulation_parameters 
    simulation_parameters.attrs['dimensionality']=3  #what happens if I set 2

    #-----
    #FIXME
    #I'm quite sure of this but not completely
    simulation_parameters.attrs['domain_dimensions'] = \
            np.array([len(r), theta_dim, phi_dim], dtype=np.int64)
    #-----

    simulation_parameters.attrs['current_time']=time
    simulation_parameters.attrs['domain_left_edge']= \
            np.array([np.min(r), np.min(theta), np.min(phi)])
    simulation_parameters.attrs['domain_right_edge']=  \
            np.array([np.max(r), np.max(theta), np.max(phi)])
    simulation_parameters.attrs['unique_identifier']='ZEUS_data' #can be anything
    simulation_parameters.attrs['cosmological_simulation']=0
    simulation_parameters.attrs['num_ghost_zones']=0
    simulation_parameters.attrs['field_ordering']=0
    simulation_parameters.attrs['geometry']=3

    #put this for now
    simulation_parameters.attrs['boundary_conditions']= \
            np.zeros(6, dtype=np.int64)

    #-----
    #FIXME
    #super wild guess, I don't have a clue here
    simulation_parameters.attrs['refine_by']= 2   #boh

    #super wild guess, same as before
    amr.create_dataset('grid_parent_id', \
            data=-np.ones(n_patches,dtype=np.int64))
    #-----

    #no particles
    amr.create_dataset('grid_particle_count', \
        data=np.zeros([n_patches,1],dtype=np.int64))

    amr.create_dataset('grid_dimensions', \
            data= grid_dimensions_GDF)

    amr.create_dataset('grid_left_index',data= grid_left_index_GDF)

    amr.create_dataset('grid_level',data=grid_level_GDF)

    den = amr.create_group('/field_types/density')
    T   = amr.create_group('/field_types/temperature')
    m1  = amr.create_group('/field_types/mag_field_x')
    m2  = amr.create_group('/field_types/mag_field_y')
    m3  = amr.create_group('/field_types/mag_field_z')
    v1  = amr.create_group('/field_types/velocity_x')
    v2  = amr.create_group('/field_types/velocity_y')
    v3  = amr.create_group('/field_types/velocity_z')
    en  = amr.create_group('/field_types/specific_thermal_energy')

    #for now, leave the staggering to 0...
    den.attrs['field_name']='density'
    den.attrs['staggering']=0
    den.attrs['field_to_cgs']=1.
    #den.attrs['field_units']=''

    T.attrs['field_name']='temperature'
    T.attrs['staggering']=0
    T.attrs['field_to_cgs']=1.
    #T.attrs['field_units']=''

    m1.attrs['field_name']='mag_r'
    m1.attrs['staggering']=0
    m1.attrs['field_to_cgs']=1.
    #m1.attrs['field_units']=''

    m2.attrs['field_name']='mag_theta'
    m2.attrs['staggering']=0
    m2.attrs['field_to_cgs']=1.
    #m2.attrs['field_units']=''

    m3.attrs['field_name']='mag_phi'
    m3.attrs['staggering']=0
    m3.attrs['field_to_cgs']=1.
    #m3.attrs['field_units']=''

    v1.attrs['field_name']='v1'
    v1.attrs['staggering']=0
    v1.attrs['field_to_cgs']=1.
    #v1.attrs['field_units']=''

    v2.attrs['field_name']='v2'
    v2.attrs['staggering']=0
    v2.attrs['field_to_cgs']=1.
    #v2.attrs['field_units']=''

    v3.attrs['field_name']='v3'
    v3.attrs['staggering']=0
    v3.attrs['field_to_cgs']=1.
    #v3.attrs['field_units']=''

    en.attrs['field_name']='specific internal energy'
    en.attrs['staggering']=0
    en.attrs['field_to_cgs']=1.
    #en.attrs['field_units']=''




    #write links following the specifications of the GDF format in yt
    for j in range(0,n_patches):
        initial = r_patch_index_initial[j]

        titlej='%010i' % j
        grid_title = "data/grid_"+titlej
        gridi = amr.create_group(grid_title)

        # I don't know if I have to write these...
        #they were in the sample file
        r_patch = hdffixyt['/grid/r'+str(j)][()]

        #gridi.attrs['dl']=
        gridi.attrs['left_edge']=np.array([r_patch[0],theta[0], phi[0]])
        gridi.attrs['right_edge']=np.array([r_patch[-1],theta[-1], phi[-1]])
        gridi.attrs['n_b']=np.array([patches_dim[j],theta_dim, phi_dim ])
        gridi.attrs['off']=np.array([initial, 0,0], dtype=np.int64)

        #-----
        #FIXME
        #not sure about this
        gridi.attrs['level']=grid_level_GDF[j]
        #-----

        #create external links
        amr[grid_title+'/density'] = \
                h5py.ExternalLink(direc+lista[i]+'yt','/den'+str(j))

        amr[grid_title+'/temperature'] = \
                h5py.ExternalLink(direc+lista[i]+'yt','/temp'+str(j))

        amr[grid_title+'/en'] = \
                h5py.ExternalLink(direc+lista[i]+'yt', \
                '/specific_thermal_energy'+str(j))

        amr[grid_title+'/velocity_x'] = \
                h5py.ExternalLink(direc+lista[i]+'yt','/v1'+str(j))

        amr[grid_title+'/velocity_y'] = \
                h5py.ExternalLink(direc+lista[i]+'yt','/v2'+str(j))

        amr[grid_title+'/velocity_z'] = \
                h5py.ExternalLink(direc+lista[i]+'yt','/v3'+str(j))

        amr[grid_title+'/mag_field_x'] = \
                h5py.ExternalLink(direc+lista[i]+'yt','/m1_'+str(j))

        amr[grid_title+'/mag_field_y'] = \
                h5py.ExternalLink(direc+lista[i]+'yt','/m2_'+str(j))

        amr[grid_title+'/mag_field_z'] = \
                h5py.ExternalLink(direc+lista[i]+'yt','/m3_'+str(j))


    amr.close()

hdffixyt.close()
