def locate_arr(array,value):
    idx = np.searchsorted(array, value, side="left")
    if idx == len(array):
        return idx-1
    if math.fabs(value - array[idx-1]) < math.fabs(value - array[idx]):
        return idx-1
    else:
        return idx

import numpy as np
from scipy import interpolate
import sys
import math
import h5py
import cons_zeus
import os.path

#this program builds a AMR version of the results of ZEUS

lista=sys.argv[1:]
n= len(lista)

direc=""

#read the ZEUS grid, spherical coordinates
if not os.path.isfile(direc+'hdffix'):
        sys.exit('hdffix not found')
hdffix = h5py.File(direc+'hdffix',"r")
dvl1a = hdffix['dvl1a'][()]
dvl2a = hdffix['dvl2a'][()]
x1b = hdffix['x1b'][()]
dx1b = hdffix['dx1b'][()]
x2b = hdffix['x2b'][()]
lgeom = hdffix['lgeom'][()]
ldimen= hdffix['ldimen'][()]
hdffix.close()

#stop if the simulation is not in spherical coordinates
if not (lgeom == 3):
    sys.exit('lgeom /= 3')

if not (ldimen == 2):
    sys.exit('ldimen /= 2')

#build the various patches of the r grid
low_r = x1b[0]
high_r = x1b[-1]
factor = 10.

ind =0
index_arr = np.empty(0, dtype=np.int)  #contains the indexes of the maximum original r covered by the patches
resolution_arr = np.empty(0)  #resolution of the patches
r= np.empty(0)                #radial patches, all concatenated in a single array
patch_len = np.empty(0, dtype=np.int)       #lenght of a sigle patch
r_patch_index_initial = np.empty(0, dtype=np.int)       #lenght of a sigle patch
r_patch_index_final = np.empty(0, dtype=np.int)       #lenght of a sigle patch

j=0
resolution = dx1b[0]/2.
stop_cycle = 0
#it is going to be the minimum in the cycle
new_max_r = x1b[0]

#analyse the grid and build the various patches in the r direction
while True:

    #increase the resolution by 2
    resolution *= 2.

    #find where the radial resolution is factor times the central radial resolution
    ind = locate_arr(dx1b, factor * resolution)

    #stop when we reach the end of x1b
    if (ind >= len(x1b)-1):
        stop_cycle=1
        ind = len(x1b)-1 

    #the new minimum is the old maximum
    new_min_r = new_max_r
    resolution_arr = np.append(resolution_arr, resolution)

    index_arr = np.append(index_arr, ind)

    #this is the new max
    new_max_r = x1b[ind]

    #calculate the number of steps, I want to ensure that the r patch has 
    #starts and ends at new_min_r and new_max_r. This will change the 
    #resolution, by using ceil I increase a little bit the real resolution
    steps_num = np.ceil((new_max_r - new_min_r)/resolution)

    #build the new r array, equally spaced
    newr = np.linspace(new_min_r, new_max_r, steps_num)
    patch_len = np.append(patch_len, len(newr))

    r_patch_index_initial = np.append(r_patch_index_initial , len(r))
    r=np.append(r,newr)
    r_patch_index_final = np.append(r_patch_index_final, len(r))

    if (stop_cycle == 1):
        break


    #safaty stop, use 100 patches at maximum
    j=j+1
    if (j > 100):
        print "no more than 100 patches are allowed (you can increase this number)"
        break


print "number of patches       =", len(index_arr)
print "patches dimensions      =", patch_len
print "total dimension         =", len(r)
print "patches resolutions (pc)=", resolution_arr/cons_zeus.cmpc


#since the ZEUS simulation is 2D axisymmetric, I will copy the arrays in 
#the third dimension. Not efficient at all, but it's a starting point
nphi = 10
#create the phi grid
phi = np.linspace(0., 2.*np.pi, nphi)

#save everything that does not depend on time, e.g. the grid
hdffixyt = h5py.File(direc+'hdffix_yt','w')

hdffixyt.create_dataset('n_patches', data=len(index_arr))
hdffixyt.create_dataset('patches_dim', data=patch_len)
hdffixyt.create_dataset('r_patch_index_initial', data=r_patch_index_initial)
hdffixyt.create_dataset('r_patch_index_final', data=r_patch_index_final)
hdffixyt.create_dataset('/grid/r_total', data=r)
hdffixyt.create_dataset('/grid/theta', data=x2b)
hdffixyt.create_dataset('/grid/phi', data=phi)

#loop over the patches
for i in range(0,len(index_arr)):
    initial = r_patch_index_initial[i]
    final   = r_patch_index_final[i]

    hdffixyt.create_dataset('/grid/r'+str(i), data=r[initial:final])
    hdffixyt.create_dataset('zero'+str(i), \
            data=np.zeros((len(r[initial:final]), len(x2b), len(phi))))

hdffixyt.close()


#main loop
for j in range(0,n):
    hydro = h5py.File(direc+lista[j],"r")
    time = hydro['time'][()]

    #covert the hydro time in yr adding 2 Gyr
    time+=2.
    time*=1e9

    #--------------------
    #read the original data and traspose them, since ZEUS 
    #has a different ordering
    e_orig = hydro[' gas energy'][()]
    e = e_orig[0,:,:]
    e = e.T
    del e_orig

    d_orig = hydro['gas density'][()]
    d = d_orig[0,:,:]
    d = d.T
    del d_orig

    T_orig = hydro['temperature'][()]
    T = T_orig[0,:,:]
    T = T.T
    del T_orig

    v1_orig = hydro[' i velocity'][()]
    v1 = v1_orig[0,:,:]
    v1 = v1.T
    del v1_orig

    v2_orig = hydro[' j velocity'][()]
    v2 = v2_orig[0,:,:]
    v2 = v2.T
    del v2_orig

    v3_orig = hydro[' k velocity'][()]
    v3 = v3_orig[0,:,:]
    v3 = v3.T
    del v3_orig
    #--------------------

    #close the hydro file
    hydro.close()


    #loop over the grids, interpolate and dump
    hydroyt = h5py.File(direc+lista[j]+'yt',"w")
    hydroyt.create_dataset('time', data=time)

    for i in range(0,len(index_arr)):
        initial = r_patch_index_initial[i]
        final   = r_patch_index_final[i]

        spline = interpolate.RectBivariateSpline(x1b, x2b, d,  kx=1, ky=1)
        rho = spline(r[initial:final], x2b)

        new2 = np.empty(shape=(len(r[initial:final]),len(x2b), len(phi)))
        for k in range(0, len(phi)):
            new2[:,:,k] =  rho
        hydroyt.create_dataset('den'+str(i), data=new2)

        #yt wants the energy per unit mass, I have per unit volume, so
        #divide by the density
        spline = interpolate.RectBivariateSpline(x1b, x2b, e,  kx=1, ky=1)
        new = spline(r[initial:final], x2b)
        new /= rho
        for k in range(0, len(phi)):
            new2[:,:,k] =  new
        hydroyt.create_dataset('en'+str(i), data=new2)

        spline = interpolate.RectBivariateSpline(x1b, x2b, T,  kx=1, ky=1)
        new = spline(r[initial:final], x2b)
        for k in range(0, len(phi)):
            new2[:,:,k] =  new
        hydroyt.create_dataset('temp'+str(i), data=new2)

        spline = interpolate.RectBivariateSpline(x1b, x2b, v1,  kx=1, ky=1)
        new = spline(r[initial:final], x2b)
        for k in range(0, len(phi)):
            new2[:,:,k] =  new
        hydroyt.create_dataset('v1'+str(i), data=new2)

        spline = interpolate.RectBivariateSpline(x1b, x2b, v2,  kx=1, ky=1)
        new = spline(r[initial:final], x2b)
        for k in range(0, len(phi)):
            new2[:,:,k] =  new
        hydroyt.create_dataset('v2'+str(i), data=new2)

        spline = interpolate.RectBivariateSpline(x1b, x2b, v3,  kx=1, ky=1)
        new = spline(r[initial:final], x2b)
        for k in range(0, len(phi)):
            new2[:,:,k] =  new
        hydroyt.create_dataset('v3'+str(i), data=new2)


        #for all the zero fields, link to the zero fields stored in hdffix_yt
        #I don't have magnetic field
        hydroyt['m1_'+str(i)] = \
                h5py.ExternalLink(direc+'hdffix_yt', 'zero'+str(i))
        hydroyt['m2_'+str(i)] = \
                h5py.ExternalLink(direc+'hdffix_yt', 'zero'+str(i))
        hydroyt['m3_'+str(i)] = \
                h5py.ExternalLink(direc+'hdffix_yt', 'zero'+str(i))



    hydroyt.close()



